var ImageModel = Backbone.Model.extend({
    initialize: function (attrs, options) {
        this.set("id", this.id);
    }
});

var ImageCollection = Backbone.Collection.extend({
    model: ImageModel,
    url: '/images'
});

var ApplicationModel = Backbone.Model.extend({
    initialize: function (attrs, options) {
        this.images = new ImageCollection();
    }
});

var ImageView = Backbone.View.extend({
    tagName: "li",
    template: _.template('<a id="image_<%=id%>" href="#"><img src="<%=imageURL%>"/><h3>images/<%=id%>.jpg</h3></a>'),
    events: {
        "click a": "onClick"
    },
    onClick: function (evt) {
        var model = this.model;
        $("#detailPage h1").html(model.get('id'));
        $("#detailPage img").attr("src", model.get('imageURL'));
        $("#deleteButton").unbind();
        $("#deleteButton").click(function (e) {
            model.destroy();
            $.mobile.changePage($("#home"), {changeHash: false});
        });
        $.mobile.changePage($("#detailPage"), {changeHash: false});
    },
    render: function () {
        $(this.el).html(this.template(this.model.toJSON()));
        return this;
    }
});

var ApplicationView = Backbone.View.extend({
    el: "#home",
    initialize: function () {
        var self = this;
        this.model.images.bind("reset", this.resetItems, this);
        this.model.images.bind("add", this.appendItem, this);
        this.model.images.bind("remove", this.removeItem, this);
        this.authState();

        $('input[type=file]').change(function (e) {
            var xhr = new XMLHttpRequest(),
                fileList = $('input[type=file]').get(0).files;
            xhr.open("POST", "/upload", true);
            xhr.setRequestHeader("Content-Type", "application/octet-stream");
            xhr.send(fileList[0]);
            self.model.images.fetch();
        });

        $("#openDialog a").click(function (e) {
            $.ajax({
                type: "POST",
                url: "/auth",
                dataType: "text",
                data: {user: $("input[type=text]").val(), password: $("input[type=password]").val()},
                success: function (data) {
                    if (data == "true") {
                        self.authState();
                    } else {
                        alert("invalid user name or password");
                    }
                },
                error: function (XMLHttpRequest, textStatus, errorThrown) {
                    console.log(errorThrown);
                }
            });
        });

        $("#logoutButton").click(function (e) {
            $.ajax({
                type: "DELETE",
                url: "/auth",
                dataType: "text",
                success: function (data) {
                    self.authState();
                }
            });
        });
    },
    authState: function () {
        var self = this;
        $.ajax({
            type: "GET",
            url: "/auth",
            dataType: "text",
            success: function (data) {
                if (data == "true") {
                    var uploadButton = $('<a data-icon="check" data-role="button" data-theme="b">Upload .JPEG File</a>');
                    uploadButton.click(function (e) {
                        $('input[type=file]').get(0).click();
                    });
                    $("#buttonArea").empty();
                    $("#buttonArea").append(uploadButton);
                    $("#logoutButton").css("visibility", "visible");
                    $("#logoutButton").button();
                    uploadButton.button();
                    self.model.images.fetch();
                } else {
                    var loginButton = $('<a href="#openDialog" data-role="button" data-icon="check" data-theme="b">Login</a>');
                    $("#buttonArea").empty();
                    $("#buttonArea").append(loginButton);
                    loginButton.button();
                    $('ul[data-role=listview]').empty();
                    $("#logoutButton").css("visibility", "hidden");

                }
            }
        });
    },
    resetItems: function (collection) {
        this.ul.empty();
        collection.each(function (model) {
            this.appendItem(model);
        }, this);
    },
    removeItem: function (model) {
        console.log(model);
        $("#image_" + model.get('id')).remove();
        $("ul[data-role=listview]").listview('refresh');
    },
    appendItem: function (model) {
        var view = new ImageView({model: model});
        $("ul[data-role=listview]").append(view.render().el);
        $("ul[data-role=listview]").listview('refresh');
    }
});


$("#home").on("pageinit", function (evt) {
    var model = new ApplicationModel({});
    var view = new ApplicationView({model: model});
});
